/**
 * Created by wilsonshang on 16-7-19.
 */

var UI = {};

var EVENT = {

	KEY_UP: "keyup",
	KEY_DOWN: "keydown",

	MOUSE_DOWN: "mousedown",
	MOUSE_MOVE: "mousemove",
	MOUSE_UP: "mouseup",
	MOUSE_OVER: "mouseover",
	MOUSE_OUT: "mouseout",
	MOUSE_WHEEL: "mousewheel",

	CLICK: "click",
	DOUBLE_CLICK: "dblclick",
	CHANGE: "change",
	TOUCH_START: "touchstart",
	TOUCH_MOVE: "touchmove",
	TOUCH_END: "touchend",

	SCROLL: "scroll"

};

/**
 * @author wilson
 * @description 元素基类:封装html原始标签,视为可显示对象
 *
 * @constructor
 *
 * @param {HtmlElement} [dom] - 代表html元素
 * */
UI.Element = function ( dom ) {

	this.dom = dom;

};

UI.Element.prototype = {

	/**
	 * @description: 添加子显示对象
	 * @param {UI.Element} argments - 一个或者多个子显示对象
	 * @return {UI.Element} 自身
	 * */
	add: function () {

		for ( var i = 0; i < arguments.length; i ++ ) {

			var argument = arguments[ i ];

			if ( argument instanceof UI.Element ) {

				this.dom.appendChild( argument.dom );

			} else {

				console.error( 'UI.Element:', argument, 'is not an instance of UI.Element.' );

			}

		}

		return this;

	},

	/**
	 * @description 删除子显示对象
	 * @param {UI.Element} argments - 一个或者多个子显示对象
	 * @return {UI.Element} 自身
	 * */
	remove: function () {

		for ( var i = 0; i < arguments.length; i ++ ) {

			var argument = arguments[ i ];

			if ( argument instanceof UI.Element ) {

				this.dom.removeChild( argument.dom );

			} else {

				console.error( 'UI.Element:', argument, 'is not an instance of UI.Element.' );

			}

		}

		return this;

	},

	/**
	 * @description 清除所有子显示对象
	 * */
	clear: function () {

		while ( this.dom.children.length ) {

			this.dom.removeChild( this.dom.lastChild );

		}

	},

	/**
	 * @description 设置dom的唯一标识
	 * @param {string} id - 唯一标识
	 * @return {UI.Element} 自身
	 * */
	setId: function ( id ) {

		this.dom.id = id;

		return this;

	},

	/**
	 * @description 设置类名
	 * @param {string} name - 类名
	 * @return {UI.Element} 自身
	 * */
	setClass: function ( name ) {

		this.dom.className = name;

		return this;

	},

	/**
	 * @description 添加css样式类
	 * @param {string} argments - 一个或者多个css样式类
	 * @return {UI.ELement} 自身
	 * */
	addClass: function () {

		for (var i = 0; i < arguments.length; ++i) {

			this.dom.classList.add(arguments[i]);

		}
		return this;

	},

	/**
	 * @description 移除css样式类
	 * @param {string} argments - 一个或者多个css样式类
	 * @return {UI.ELement} 自身
	 * */
	removeClass: function () {

		for (var i = 0; i < arguments.length; ++i) {

			this.dom.classList.remove(arguments[i])

		}

		return this;

	},

	/**
	 * @description 设置html元素制定样式
	 * @param {string} style - css属性名称
	 * @param {string} array - css属性值
	 * @return {UI.Element} 自身
	 * */
	setStyle: function ( style, array ) {

		for ( var i = 0; i < array.length; i ++ ) {

			this.dom.style[ style ] = array[ i ];

		}

		return this;

	},

	/**
	 * @description 设置是否禁用html元素标签
	 * @param {boolean} value - 禁用标识位
	 * @return {UI.Element} 自身
	 * */
	setDisabled: function ( value ) {

		this.dom.disabled = value;

		return this;

	},

	/**
	 * @description 设置html标签文本类容
	 * @param {string} value - 文本内容
	 * @return {UI.Element} 自身
	 * */
	setTextContent: function ( value ) {

		this.dom.textContent = value;

		return this;

	},

	/**
	 * @description 获取对象的宽度(包括内边距和边框,不包括外边距)
	 * @return {number} 宽度
	 * */
	getWidth: function () {

		var boundRect = this.dom.getBoundingClientRect();

		return boundRect.width;

	},

	/**
	 * @description 获取对象的高度(包括内边距和边框,不包括外边距)
	 * @return {number} 高度
	 * */
	getHeight: function () {

		var boundRect = this.dom.getBoundingClientRect();

		return boundRect.height;

	},

	/**
	 * @description 获取对象的偏移量(包括内边距和边框,不包括外边距)
	 * @return {object} left和top偏移量
	 * */
	getOffset: function () {

		var boundRect = this.dom.getBoundingClientRect();

		return {left: boundRect.left, top: boundRect.top};

	},

	/**
	 * @description 添加监听函数(非捕获阶段)
	 * @param {string} event - 事件类别:EVENT枚举中
	 * @param {function} listener - 监听函数
	 * @param {UI.Element} [scope] - 调用域,默认为被监听对象
	 * @return {UI.Element} 自身
	 * */
	addEventListener: function (event, listener, scope) {

		this.dom.addEventListener( event, listener.bind( scope || this ), false );
		return this;

	},

	/**
	 * @description 移除监听函数(非捕获阶段)
	 * @param {string} event - 事件类别:EVENT枚举中
	 * @param {function} listener - 监听函数
	 * @param {UI.Element} [scope] - 调用域,默认为被监听对象
	 * @return {UI.Element} 自身
	 * */
	removeEventListener: function (event, listener, owner) {

		this.dom.removeEventListener( event, listener.bind( scope || this ), false );
		return this;

	},

};

var properties = [ 'position', 'left', 'top', 'right', 'bottom', 'width', 'height', 'border', 'borderLeft',
'borderTop', 'borderRight', 'borderBottom', 'borderColor', 'display', 'overflow', 'margin', 'marginLeft', 'marginTop', 'marginRight', 'marginBottom', 'padding', 'paddingLeft', 'paddingTop', 'paddingRight', 'paddingBottom', 'color',
'background', 'backgroundColor', 'opacity', 'fontFamily', 'fontSize', 'fontWeight', 'textAlign', 'textDecoration', 'textTransform', 'cursor', 'zIndex',
'visibility', 'float', "borderRadius", "clear"];

properties.forEach( function ( property ) {

	var method = 'set' + property.substr( 0, 1 ).toUpperCase() + property.substr( 1, property.length );

	UI.Element.prototype[ method ] = function () {

		this.setStyle( property, arguments );

		return this;

	};

} );


// events

var events = [ 'KeyUp', 'KeyDown', 'MouseOver', 'MouseOut', 'Click', 'DblClick', 'Change', 'TouchStart', 'TouchMove', 'TouchEnd' ];

events.forEach( function ( event ) {

	var method = 'on' + event;

	UI.Element.prototype[ method ] = function ( callback ) {

		this.dom.addEventListener( event.toLowerCase(), callback.bind( this ), false );
		return this;

	};

} );

events.forEach( function ( event ) {

	var method = 'remove' + event;

	UI.Element.prototype[ method ] = function ( callback ) {

		this.dom.removeEventListener( event.toLowerCase(), callback.bind( this ), false );

		return this;

	};

} );


/**
 * @author wilson
 * @decription 普通容器
 * @constructor
 * */
UI.Div = function () {

	UI.Element.call( this );

	this.dom = document.createElement( 'div' );

	return this;

};

UI.Div.prototype = Object.create( UI.Element.prototype );
UI.Div.prototype.constructor = UI.Div;

/**
 * @author wilson
 * @decription 有序列表
 * @constructor
 * */
UI.List = function () {

	UI.Element.call(this);

	var dom = document.createElement( 'ol' );
	dom.className = 'List';
	dom.style.listStyle = "none";

	this.dom = dom;

	return this;

};

UI.List.prototype = Object.create( UI.Element.prototype );
UI.List.prototype.constructor = UI.List;

/**
 * @author wilson
 * @description 列表项
 * @constructor
 * */
UI.ListItem = function () {

	UI.Element.call(this);

	var dom = document.createElement( 'li' );
	dom.className = 'ListItem';

	this.dom = dom;

	return this;

};

UI.ListItem.prototype = Object.create( UI.Element.prototype );
UI.ListItem.prototype.constructor = UI.ListItem;

/**
 * @author wilson
 * @description 普通面板
 * @constructor
 * */
UI.Panel = function () {

	UI.Element.call( this );

	var dom = document.createElement( 'div' );
	dom.className = 'Panel';

	this.dom = dom;

	return this;

};

UI.Panel.prototype = Object.create( UI.Element.prototype );
UI.Panel.prototype.constructor = UI.Panel;

/**
 * @author wilson
 * @description 简单文本
 * @constructor
 * */
UI.Text = function ( text ) {

	UI.Element.call( this );

	var dom = document.createElement( 'span' );
	dom.className = 'Text';
	this.dom = dom;
	this.setValue( text );

	return this;

};

UI.Text.prototype = Object.create( UI.Element.prototype );
UI.Text.prototype.constructor = UI.Text;

/**
 * @description 返回文本内容
 * @return {string} 文本内容
 * */
UI.Text.prototype.getValue = function () {

	return this.dom.textContent;

};

/**
 * @description 返回文本内容
 * @return {UI.Text} 自身
 * */
UI.Text.prototype.setValue = function ( value ) {

	if ( value !== undefined ) {

		this.dom.textContent = value;

	}

	return this;

};

/**
 * @author wilson
 * @description 换行
 * @constructor
 * */
UI.Break = function () {

	UI.Element.call( this );

	var dom = document.createElement( 'br' );
	dom.className = 'Break';

	this.dom = dom;

	return this;

};

UI.Break.prototype = Object.create( UI.Element.prototype );
UI.Break.prototype.constructor = UI.Break;

/**
 * @author wilson
 * @description: 水平分割线
 * @constructor
 * */
UI.HorizontalRule = function () {

	UI.Element.call(this);

	var dom = document.createElement("hr");
	this.dom = dom;
	dom.className = 'HorizontalRule';
	this.addClass("divide-line");

};

UI.HorizontalRule.prototype = Object.create(UI.Element.prototype);
UI.HorizontalRule.prototype.constructor = UI.HorizontalRule;

// Button

UI.Button = function ( value ) {

	UI.Element.call( this );

	var scope = this;

	var dom = document.createElement( 'button' );
	dom.className = 'Button';

	this.dom = dom;
	this.dom.textContent = value;

	return this;

};

UI.Button.prototype = Object.create( UI.Element.prototype );
UI.Button.prototype.constructor = UI.Button;

UI.Button.prototype.setLabel = function ( value ) {

	this.dom.textContent = value;

	return this;

};

//兼容安卓不识别assign的问题 
if (typeof Object.assign != 'function') {
    Object.assign = function (target) {
    //第一个传参不能是undefined和null，因为它们不能被转为对象
      if (target === undefined || target === null) {
        throw new TypeError('Can not convert undefined or null to object');
      }
    //使用Object对象化target
      var output = Object(target);
      for (var idx = 1,l=arguments.length; idx < l; idx++) {
        var source = arguments[idx];
        //后续传参也需要判断undefined和null
        if (source !== undefined && source !== null) {
          for (var key in source) {
            if (Object.prototype.hasOwnProperty.call(source, key)) {
              output[key] = source[key];
            }
          }
        }
      }
      return output;
    };
}