/**
 * Created by wilsonshang on 16-7-20.
 */

/**
 * @author wilson
 * @description 日历"天"元素
 * @constructor
 * @param {UI.MonthPanel} monthPanel - 父类面板引用
 * @param {object} [parameters] - 携带数据: date:日期; info:额外信息; disabled:是否禁用; status:状态
 * */
UI.DateItem = function (monthPanel, parameters) {

    this.monthPanel = monthPanel;
    parameters = parameters || {};

    if (!parameters.hasOwnProperty("date")) parameters.date = new Date();
    if (!parameters.hasOwnProperty("info")) parameters.info = "入住";
    if (!parameters.hasOwnProperty("disabled")) parameters.disabled = false;
    if (!parameters.hasOwnProperty("status")) parameters.status = DateUtils.DATE_ITEM_NORMAL;
    this.parameters = parameters;

    UI.Element.call(this);
    var dom = document.createElement("div");
    dom.className = "DateItem";
    this.dom = dom;

    //日期文本
    var dateTxt = new UI.Text(parameters.date.getDate().toString());
    this.add(dateTxt);
    dateTxt.addClass("date-date");

    this.dateTxt = dateTxt;

    //额外信息
    var infoTxt = new UI.Text(parameters.info);
    this.add(infoTxt);

    infoTxt.addClass("date-info");

    this.infoTxt = infoTxt;

    this.setWidth(DateUtils.colWidthRatio + "%");
    this.setFloat("left");

    return this;

};

UI.DateItem.prototype = Object.create(UI.Element.prototype);

Object.assign(UI.DateItem.prototype, {

    constructor: UI.DateItem,

    /**
     * @description 返回日期
     * @return {date} 日期
     * */
    getDate: function () {

        return this.parameters.date;

    },

    /**
     * @description 返回日期中文显示
     * @return {string} 日期
     * */
    toMMDD: function () {

        var date = this.getDate();

        return (date.getMonth() + 1) + "月" +  date.getDate() + "日";

    },

    /**
     * @description 设置日期
     * @param {Date} date - 设置日期
     * @return {UI.DateItem} 自身
     * */
    setDate: function (date) {

        this.parameters.date = date;

        this.dateTxt.setValue(date.getDate().toString());

        return this;

    },

    /**
     * @description 设置额外信息
     * @param {string} info - 显示信息
     * @return {UI.DateItem} 自身
     * */
    setInfo: function (info) {

        this.parameters.info = info;

        if (info === null || info === "") {

            this.infoTxt.setVisibility("hidden");

        } else {

            this.infoTxt.setVisibility("visible");

            this.infoTxt.setValue(info);

        }

        return this;

    },

    /**
     * @description 设置水平偏移量
     * @param {int} weekDay - 星期几?
     * @return {UI.DateItem} 自身
     * */
    setHorizontalOffset: function (weekDay) {

        this.setMarginLeft(DateUtils.colWidthRatio * weekDay + "%");

        return this;

    },

    /**
     * @description 设置状态
     * @param {int} status - 状态(DateUtil.DATE_ITEM_INVALID; DateUtils.DATE_ITEM_NORMAL;DateUtils.DATE_ITEM_SELECTED;DateUtils.DATE_ITEM_NODE)
     * @return {UI.DateItem} 自身
     * */
    setStatus: function (status) {

        this.status = status;


        switch (status) {

            case DateUtils.DATE_ITEM_INVALID:

                this.setColor("#A4A4A4");

                break;

            case DateUtils.DATE_ITEM_NORMAL:

                this.dateTxt.setColor("#1C1C1C");

                this.dateTxt.setBackgroundColor("transparent");

                break;

            case DateUtils.DATE_ITEM_SELECTED:

                this.dateTxt.setColor("#0085F0");
                this.dateTxt.setBackgroundColor("transparent");

                break;

            case DateUtils.DATE_ITEM_NODE:

                this.dateTxt.setValue(this.parameters.date.getDate());

                this.dateTxt.setColor("#FFFFFF");
                this.dateTxt.setBackgroundColor("#0085F0");

                break;


            default:

                console.log("DateItem unknow status");

                break;

        }

        return this;

    },

    /**
     * @description 设置是否被禁用
     * @param {boolean} value - 是否被禁用标识
     * @return {UI.DateItem} 自身
     * */
    setDisabled: function (value) {

        this.parameters.disabled = value;

        UI.Element.prototype.setDisabled.call(this, value);

        if (value) {

            this.setStatus(DateUtils.DATE_ITEM_INVALID);
            this.removeClick(this.tapHandler);

        } else {

            this.setStatus(DateUtils.DATE_ITEM_NORMAL);
            this.onClick(this.tapHandler);

        }

        return this;

    },

    /**
     * @description 点击时间处理函数
     * @param {Event} event - 事件
     * */
    tapHandler: function (event) {

        event.stopImmediatePropagation();

        this.monthPanel.tapHandler(this);

    }

});