/**
 * Created by wilsonshang on 16-7-20.
 */

/**
 * @description 酒店日历选择面板
 * @constructor
 * @param {function} callback - 选择完回调函数
 *
 * */
UI.HotelDatePanel = function ( callback) {

    UI.DatePanel.call(this,callback);

    this.timestampMin = -1;
    this.timestampMax = -1;

    this.refMonths = [];

    this.tip = new UI.HotelTip();

    this.add(this.tip);
    this.tip.setAlpha(0);

    this.startItem = null;
    this.endItem = null;

    this.timeOutID = 0;

    this.startInfo = "入住";
    this.endInfo = "离店";


};

UI.HotelDatePanel.prototype = Object.create(UI.DatePanel.prototype);

Object.assign(UI.HotelDatePanel.prototype, {

    constructor: UI.HotelDatePanel,


    /**
     * @description 设置选择区间(闭区间)同时改变显示状态
     * @param {Date} minDate - 起始日期
     * @param {Date} maxDate - 终止日期
     * @return {UI.HotelDatePanel} - 自身
     *
     * */
    setSelect: function (minDate, maxDate) {

        this.timestampMin = minDate.getTime();
        this.timestampMax = maxDate.getTime();
        this.updateSelect();
        return this;

    },

    setStartInfo: function (startInfo) {

        this.startInfo = startInfo;

        if (this.startItem == null) return;

        this.startItem.setInfo(startInfo);

    },

    setEndInfo: function (endInfo) {

        this.endInfo = endInfo;

        if (this.endItem == null) return;

        this.endItem.setInfo(endInfo);

    },

    /**
     * @description 更新选择区间显示状态
     * @return {int} - 返回天数
     *
     * */
    updateSelect: function () {

        this.clearRefMonths();

        var minDate = new Date(this.timestampMin);
        var maxDate = new Date(this.timestampMax);

        var startYear = minDate.getFullYear();
        var startMonth = minDate.getMonth();
        var startDate = minDate.getDate();

        var endYear = maxDate.getFullYear();
        var endMonth = maxDate.getMonth();
        var endDate = maxDate.getDate();

        var monthAmount = (endYear - startYear) * 12 - startMonth + endMonth + 1;

        var year, month, dateMin, dateMax, monthPanel;


        var c = 0;

        var r;

        for (var i = 0; i < monthAmount; ++i) {

            year = startYear + Math.floor((startMonth + i) / 12);
            month = (startMonth + i) % 12;

            monthPanel = this.getMonthPanel(year, month);

            this.refMonths.push(monthPanel);

            dateMin = (year === startYear && month === startMonth) ? startDate : 1;
            dateMax = (year === endYear && month == endMonth) ? endDate : monthPanel.dayAmount;

            r = monthPanel.setItemsSelected(dateMin, dateMax);

            if (i === 0 && this.startItem === null) {

                this.startItem = r[0];

            }

            if (i === monthAmount - 1 && this.endItem === null) {

                this.endItem = r[1];

            }

            //c += (dateMax - dateMin);

        }

        c = (this.timestampMax - this.timestampMin) / (3600 *1000 * 24);

        if (this.startItem !== null && this.startItem !== undefined) {

            this.startItem.setStatus(DateUtils.DATE_ITEM_NODE);
            this.setStartInfo(this.startInfo);

        }

        if (this.endItem !== null && this.endItem !== undefined) {

            this.endItem.setStatus(DateUtils.DATE_ITEM_NODE);
            this.setEndInfo(this.endInfo);

        }

        return c;

    },

    /**
     * @description 点击处理函数
     * @param {UI.DateItem} - 点击事件源
     * */
    tapHandler: function (dateItem) {

        clearTimeout(this.timeOutID);

        this.tip.setAlpha(0.0, false);
        this.tip.setAlpha(1.0, true);

        var scope = this;

        this.timeOutID = setTimeout(function () {

            scope.tip.setAlpha(0.0, true);

        }, 2000);


        var itemWidth = dateItem.getWidth();
        var itemHeight = dateItem.getHeight();

        var offset = dateItem.getOffset();


        var offsetX = offset.left + (itemWidth - this.tip.getWidth()) / 2;
        var offsetY = offset.top - itemHeight + window.scrollY;

        this.tip.setShow(offsetX, offsetY);

        var timestamp = dateItem.getDate().getTime();

        var monthPanel;

        //选择开始
        if ((this.timestampMin === -1 || this.timestampMin >= timestamp) || //首次选择时间点或者选择的时间点小于最小时间戳

            (this.timestampMin !== -1 && this.timestampMax !== -1 && this.timestampMax > this.timestampMin) //已存在起始结束时间点后的选择也视为选择的开始

        ) {

            this.timestampMin = this.timestampMax = timestamp;

            this.clearRefMonths();

            if (this.startItem !== null) {

                this.startItem.setStatus(DateUtils.DATE_ITEM_NORMAL);
                this.startItem.setInfo("");

            }

            this.startItem = dateItem;

            dateItem.setStatus(DateUtils.DATE_ITEM_NODE);
            dateItem.setInfo(this.startInfo);

            this.tip.setTitle("请选择" + this.endInfo + "日期");
            this.tip.setContent(dateItem.toMMDD() + this.startInfo);

        } else {

            //选择结束
            this.endItem = dateItem;

            this.timestampMax = timestamp;

            var c = this.updateSelect();
         
            var minDate = new Date(this.timestampMin);
            var maxDate = new Date(this.timestampMax);

            this.tip.setTitle("共" + c + "晚");
            this.tip.setContent(this.dateToMMDD(minDate) + "-" + this.dateToMMDD(maxDate));

            if (this.callback != null) this.callback();

        }

    },

    /**
     * @description 日期转换为固定的中文格式
     * @param {Date} date - 日期
     * @return {string} -中文显示
     * */
    dateToMMDD: function (date) {

        return (date.getMonth() + 1) + "月" +  date.getDate() + "日";

    },


    /**
     * @description 清除选择区间所关联的月份以及选择状态
     *
     * */
    clearRefMonths: function () {

        this.refMonths.forEach(function(mp) {

            mp.clearSelect();

        });

        this.refMonths.length = 0;

    },

    /**
     * @description 清除所有数据
     * */
    clear: function () {

        this.clearRefMonths();
        this.startItem = null;
        this.endItem = null;
        this.timestampMin = -1;
        this.timestampMax = -1;

    }


});